!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_shapef
!! NAME 
!! calc_shapef
!!
!! FUNCTION
!! Compute normalized shape function and its moments (gnorm)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pawdata
!!    %rad(max_meshsz)= Coordinates of the radial grid
!!  pshead
!!    %lambda= Lambda in gaussian type g(r)
!!    %l_size= Max. value of l+1 leading to a non zero Gaunt coeffs
!!    %rad_step= Step corresponding to radial mesh
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type
!!    %sigma= Sigma for gaussian type g(r)
!!    %sph_meshsz= Dimension of radial mesh for PAW spheres
!!
!! OUTPUT
!!  pawarray
!!    %gnorm(l_size)= Normalisation of shape function for each l
!!    %shapefunc(sph_meshsz)= Normalized shape function
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!      csimp
!!
!! SOURCE

 subroutine calc_shapef(pawarray,pawdata,pshead)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ------------------------------------
 type(pawarray_type) :: pawarray
 type(pawdata_type)  :: pawdata
 type(pshead_type)   :: pshead

!Local variables-------------------------------
 integer :: il,ir
 real(dp) :: arg,shapefunc1,shapefunc2
 real(dp), allocatable :: ff(:)

!Statement functions -----------------------------------
 shapefunc1(arg)= exp(-(arg/pshead%sigma)**pshead%lambda)
 shapefunc2(arg)= (sin(pi*arg/pshead%rc_sph)/(pi*arg/pshead%rc_sph))**2

! *************************************************************************

!Select the suitable shape function
 pawarray%shapefunc(1)=1.d0
 if (pshead%shape_type==1) then
  do ir=2,pshead%sph_meshsz
   pawarray%shapefunc(ir)=shapefunc1(pawdata%rad(ir))
  enddo
 else if (pshead%shape_type==2) then
  do ir=2,pshead%sph_meshsz
   pawarray%shapefunc(ir)=shapefunc2(pawdata%rad(ir))
  enddo
 else
  print *,"Error in calc_shapef: invalid value of pshead%shape_type !"
  stop
 endif

!Normalize shape functions
 allocate(ff(pshead%sph_meshsz));pawarray%gnorm=zero;ff(1)=zero
 do il=1,pshead%l_size
  ff(2:pshead%sph_meshsz)=pawarray%shapefunc(2:pshead%sph_meshsz)*pawdata%rad(2:pshead%sph_meshsz)**(2*il)
  call csimp(ff,pshead%sph_meshsz,pshead%rad_step,pawarray%gnorm(il))
  pawarray%gnorm(il)=1.d0/pawarray%gnorm(il)
 enddo
 deallocate(ff)

end subroutine
!!***
