!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_vhtnzc
!! NAME
!! calc_vhtnzc
!! 
!! FUNCTION
!! Compute vh(tnZc) and the <phi_i|vh(tnZc)|phi_j> matrix elements
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pawarray
!!    %gnorm(l_size)= Normalisation of shape function for each l
!!    %shapefunc(sph_meshsz)= Normalized shape function
!!    %vbare(sph_meshsz)= bare local potential (part of VH(tnzc))
!!  pawdata
!!    %indlmn(6,lmn_size)= Gives l,m,n,lm,ln,s for i=lmn
!!    %rad(max_meshsz)= Coordinates of the radial grid
!!  pawps
!!    %coreden4pr2(core_meshsz)= Core density multiplied by 4Pi.r2
!!    %tcoreden4pr2(core_meshsz)= Pseudized core density multiplied by 4Pi.r2
!!    %tphi(sph_meshsz,basis_size)= PAW atomic pseudo-wavefunctions on the radial grid
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %core_charge= Core charge
!!    %core_meshsz= Dimension of radial mesh for core density
!!    %lmn_size= Number of elements for the paw basis
!!    %rad_step= Step corresponding to radial mesh
!!    %sph_meshsz= Dimension of radial mesh for PAW spheres
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhtnzc
!!
!! OUTPUT
!!  pawarray
!!    %vhtnzcij(lmn2_size)= matrix element of the Hartree potential of
!!                          the ps-density of the nucleus+core electrons
!!  pawps
!!    %vhtnzc(core_meshsz)= Hartree potential of the ps-density
!!                           of the nucleus + core electrons
!!
!! PARENTS
!!      atompaw2abinit
!!
!! CHILDREN
!!      csimp,poisson
!!
!! SOURCE

 subroutine calc_vhtnzc(pawarray,pawdata,pawps,pshead)

  use defs_pawps
  use defs_basis

  implicit none

!Arguments ---------------------------------------------
 type(pawarray_type) :: pawarray
 type(pawdata_type)  :: pawdata
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer :: ilm,ilmn,iln,jlm,jlmn,jln,j0lmn,klmn
 real(dp) :: qcore,test_vhtnzc
 real(dp),allocatable :: den(:),ff(:)
!-------------------------------------------------------

!Compute VH(tnZc)   --  read from atomicdata file
!------------------------------------------------------------------
!Compute Zcore
! allocate(ff(pshead%sph_meshsz))
! ff(1:pshead%sph_meshsz)=pawps%coreden4pr2(1:pshead%sph_meshsz)-pawps%tcoreden4pr2(1:pshead%sph_meshsz)
! call csimp(ff,pshead%sph_meshsz,pshead%rad_step,qcore)
! deallocate(ff)

!Compute r*Vh(tnZC(r)) using Poisson's equation
! allocate(den(pshead%vloc_meshsz))
! den=zero;den(1:pshead%core_meshsz)=pawps%tcoreden4pr2(1:pshead%core_meshsz)
! den(1:pshead%sph_meshsz)=den(1:pshead%sph_meshsz)+(qcore-pshead%atomic_charge) &
!&                        *pawarray%gnorm(1)*pawarray%shapefunc(1:pshead%sph_meshsz) &
!&                        *pawdata%rad(1:pshead%sph_meshsz)**2
! call poisson(den,pshead%rad_step,pshead%vloc_meshsz,pawps%vhtnzc)
! deallocate(den)

!Compute vhtnZc from r*Vh(tnZC(r))
! pawps%vhtnzc(2:pshead%vloc_meshsz)=pawps%vhtnzc(2:pshead%vloc_meshsz)/pawdata%rad(2:pshead%vloc_meshsz)
 call deducer0(pawps%vhtnzc,pshead%vloc_meshsz,pawdata%rad)

!Add Vbare
! pawps%vhtnzc(1:pshead%sph_meshsz)=pawps%vhtnzc(1:pshead%sph_meshsz)+pawarray%vbare(1:pshead%sph_meshsz)

!Compute matrix elements: vhtnZc_ij(r)=<tphi_i|vh(tnZc)|thpi_j>
!------------------------------------------------------------------
 allocate(ff(pshead%sph_meshsz))
 pawarray%vhtnzcij=zero
 do jlmn=1,pshead%lmn_size
  j0lmn=jlmn*(jlmn-1)/2
  jlm=pawdata%indlmn(4,jlmn);jln=pawdata%indlmn(5,jlmn)
  do ilmn=1,jlmn
   klmn=j0lmn+ilmn
   ilm=pawdata%indlmn(4,ilmn);iln=pawdata%indlmn(5,ilmn)
   if (jlm==ilm) then
    ff(1:pshead%sph_meshsz)=pawps%tphi(1:pshead%sph_meshsz,iln)*pawps%tphi(1:pshead%sph_meshsz,jln) &
&                         *pawps%vhtnzc(1:pshead%sph_meshsz)
    call csimp(ff,pshead%sph_meshsz,pshead%rad_step,pawarray%vhtnzcij(klmn))
   endif
  enddo
 enddo
 deallocate(ff)

!Test the behaviour of VhtnZc at long range: vhtnzc(r)=-Z/r ?
!------------------------------------------------------------
  write(6,'(/,a)') 'Info:'
  write(6,'(a,f5.2,a,a,g11.4)') &
&   '  At r_vloc=',pawdata%rad(pshead%vloc_meshsz),' a.u.,',&
&   ' VHartree(ntild(Zv+Zc))= -Zv/r + ',&
&     pawps%vhtnzc(pshead%vloc_meshsz) &
&    +(pshead%atomic_charge-pshead%core_charge)/pawdata%rad(pshead%vloc_meshsz)
  write(6,'(a)') &
&   '  This quantity must be as small as possible.'

 end subroutine calc_vhtnzc
