!{\src2tex{textfont=tt}}
!!****f* ABINIT/csimp
!! NAME 
!! csimp
!!
!! FUNCTION
!! Do integral on a regular grid using corrected Simpson rule
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  ff(meshsz)=integrand values
!!  meshsz= Dimension of radial mesh
!!  rad_step= Step corresponding to radial mesh
!!
!! OUTPUT
!!  simp=resulting integral by corrected Simpson rule
!!
!! PARENTS
!!      calc_dij0,calc_kij,calc_shapef,calc_vhnzc,calc_vhtnzc
!!
!! CHILDREN
!!
!! SOURCE

 subroutine csimp(ff,meshsz,rad_step,simp)

 use defs_basis

 implicit none

!Arguments ------------------------------------
 integer :: meshsz
 real(dp) :: rad_step,simp
 real(dp) :: ff(meshsz)

!Local variables-------------------------------
 integer :: ii
 real(dp) :: simp1,simp2,simp4

! *************************************************************************

 if (mod(meshsz,2)==1) then
  simp1=ff(1)+ff(meshsz)
  simp2=zero;simp4=ff(2)
  do ii=3,meshsz-1,2
   simp2=simp2+ff(ii);simp4=simp4+ff(ii+1)
  enddo
 else
  simp1=1.25d0*ff(1)+3.d0*ff(2)-0.25d0*ff(3)+ff(meshsz)
  simp2=zero;simp4=ff(3)
  do ii=4,meshsz-1,2
   simp2=simp2+ff(ii);simp4=simp4+ff(ii+1)
  enddo
 endif

 simp=rad_step/3.d0*(simp1+two*simp2+four*simp4)


 end subroutine
!!***
