!{\src2tex{textfont=tt}}
!!****f* ABINIT/poisson
!! NAME
!! poisson
!! 
!! FUNCTION
!! Solve poisson equation for a density given on a regular grid
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!! 
!! INPUTS
!!  den(nn) is electron density * (4*pi*r**2)
!!  hh= step of radial grid
!!  nn= dimension of the radial arrays
!!
!! OUTPUT
!!  rv(nn) is returned as electrostatic potential * r (in Hartree*Bohr!)
!!
!! PARENTS
!!      calc_vhnzc,calc_vhtnzc
!!
!! CHILDREN
!!
!! SOURCE

 subroutine poisson(den,hh,nn,rv)

  use defs_basis
  implicit none

!Arguments ------------------------------------
 integer :: nn
 real (dp) :: hh
 real (dp) :: den(nn),rv(nn)

!Local variables-------------------------------
 integer :: ir,jr
 real(dp) :: h3
 real (dp) :: aa(nn),bb(nn),cc(nn),dd(nn),rad(nn)

! *************************************************************************

  h3=hh/3.d0
  do jr=nn,2,-1
   ir=nn-jr+1
   rad(jr)=hh*dble(jr-1)
   aa(ir)=den(jr)
   bb(ir)=den(jr)/rad(jr)
  end do
  aa(nn)=2.d0*aa(nn-1)-aa(nn-2)
  bb(nn)=2.d0*bb(nn-1)-bb(nn-2)
  cc(1)=zero;dd(1)=zero
  do ir=3,nn,2
   cc(ir)  =cc(ir-2)+h3*(aa(ir-2)+4.d0*aa(ir-1)+aa(ir))
   cc(ir-1)=cc(ir-2)+h3*(1.25d0*aa(ir-2)+2.0d0*aa(ir-1)-0.25d0*aa(ir))
   dd(ir)  =dd(ir-2)+h3*(bb(ir-2)+4.d0*bb(ir-1)+bb(ir))
   dd(ir-1)=dd(ir-2)+h3*(1.25d0*bb(ir-2)+2.d0*bb(ir-1)-0.25d0*bb(ir))
  end do
  if (mod(nn,2)==0) then
   cc(nn)=cc(nn-2)+h3*(aa(nn-2)+4.d0*aa(nn-1)+aa(nn))
   dd(nn)=dd(nn-2)+h3*(bb(nn-2)+4.d0*bb(nn-1)+bb(nn))
  end if
  rv(1)=zero
  do ir=2,nn
   jr=nn-ir+1
   rv(ir)=(dd(jr)*rad(ir)+cc(nn)-cc(jr))
  end do

 end subroutine
!!***
